

////////////////////////////////////////////////////
// weller_driver_v0910.c
////////////////////////////////////////////////////
// Copyright 2015 - 2018 Jaakko Kairus
// jaakko@kair.us
//
// You may freely use and modify this software for personal use.
// Commercial use of this software or parts of it is prohibited
// without written permission from author.
// DISCLAIMER: This software comes with ABSOLUTELY NO WARRANTY
//
// For more information about this project see project web page at
// http://kair.us/projects/weller/
//
// Compiled with CCS compiler version 5.081
// Total compiled length 5937 bytes
///////////////////////////////////////////////////
// Version history
// v0.1 30.5.2015
//      Initial version. For modified kuivanapitolmmitin board.
// v0.2 4.6.2015
//      Reduce ADC averaging from 512 to 16 to speed up heater control response
//      Temperature reading is now 16 bit and not 21 bit anymore
// v0.3 3.11.2015
//		First FW version for dedicated Weller driver v1 board
//		Added menus to adjust setback temp and delay, poweroff delay, temperature
//		units and step size. Diagnostics menu to show cold point compensation temp,
//		both thermocouple temperatures separately, identified iron type, reed status
//		and correcting for reference tolerance. Moved 7-segment character defines
//		to separate header file. Changed saa_1064 function to allow displaying of
//		temperature, numeric values without decimals and reference with decimal.
//		Added a lot of messages to support the new menu structure.
// v0.4 22.12.2015
//		Cleanup for release
// v0.5 27.12.2015
//		Fixed saving temperature setting (broken because of re-organize of EEPROM addresses)
//		Remove display addressing of show_7seg_num function because only one display used
// v0.6 10.1.2017
//		Changed back to using resistor to recognize tip type to reduce buzz with WMRT.
//		Added FW version display to diagnostics menu.
//		Added menu setting 'poor'; when enabled, assumes plain WMRP tip connected w/o
//		any resistors. Useful if only used for WMRP connected to a 3.5 mm socket.
//		Added mains frequency filtering to temperature readout. Changed temperature readout
//		to happen at fixed intervals (10 ms for 50 Hz mains, 8.32 ms for 60 Hz mains).
//		Two consecutive temperature readings are averaged which filters away the mains
//		frequency. Added new menu setting to choose 50 Hz or 60 Hz. Shortening the interval
//		required different optimizations to still achieve long enough heating cycle.
//		Added filtering to KTY readout (slower response, more averaging)
//		Splitted old menuMachine() function to separate updateDispay() and menuMachine()
//		functions, now slow updateDisplay() is only called once during the heating cycle,
//		menuMachine is called in interrupt, after encoder readout. Ensures no encoder
//		events are missed.
// v0.7	31.1.2017
//		Reduced maximum heating duty cycle back to about 69% (63% at 60Hz) . Was about 86%
//		with FW 0.6 which may be too high and stress the tip. Added some sanity checks to
//		EEPROM reads and writes.
// v0.8	5.2.2017
//		Fixed time delays to use the 832 us interrupt period. Better button debounce.
//		Fixed tip detection when plugging in iron which is on stand (reed closed)
// v0.9 28.7.2017
//		Added check for all parameters which are stored to EEPROM. If any of parameters
//		are out of valid range, all of them are set to default values. This fixes problem
//		when using MPLAB IPE to program the PIC, and EEPROM is not erased or programmed.
//		Added clearing of display anodes before parameter save to avoid bright flash of
//		currently active segments.
// v0.901 28.7.2017
//		Fixed setback and poweroff incase delay set to 0 minutes. Added some fixes to
//		tip type recognition at startup to prevent errorneous WMRT detection. Added
//		workaround to allow returning from setback for five seconds even if setback
//		delay set to zero. Allows e.g. to put the iron to standby from setback with two
//		consecutive button presses.
// v0.902 27.8.2018
//		Added compile option for common cathode display. WELLER_CC_DISPLAY.
//		Length 4888 words, 5307 w/o #opt compress
// v0.903 15.9.2018
//		Added diag menu items to show PWM duty cycle values for both heaters. Added diag
//		menu setting for 'idle duty cycle', which allows station to go to setback or power
//		off based on average power to tip. Added diag menu setting to limit maximum
//		heater duty cycle. This can be used to limit max power, possibly lenghtening tip
//		life.
// v0.910 17.10.2018
//		Revised thermocouple lookup table based on actual measurements with Weller K1101
//		calibration tip. Degree sign now shows reed status, it is dimmed when reed is closed.
//		Added possibility to turn idle duty cycle to 'off'.
//v1.00  14.04.2020
//      Ported the code to atmega 16 chip  

#include <avr/io.h>
#include <avr/interrupt.h>
#define FW_VERSION 1000
// #define WELLER_CC_DISPLAY	// Uncomment this line if you use common cathode display

#ifdef WELLER_CC_DISPLAY
#define SEGMENT_A PORTC = 0b00000001;
#define SEGMENT_B PORTC = 0b00000010;
#define SEGMENT_C PORTC = 0b00000100;
#define SEGMENT_D PORTC = 0b00001000;
#define SEGMENT_E PORTC = 0b00010000;
#define SEGMENT_F PORTC = 0b00100000;
#define SEGMENT_G PORTC = 0b01000000;
#define SEGMENT_DP PORTC = 0b10000000;
#define SEGMENT_NONE PORTC = 0b00000000;
#else
#define SEGMENT_A PORTC = 0b11111110;
#define SEGMENT_B PORTC = 0b11111101;
#define SEGMENT_C PORTC = 0b11111011;
#define SEGMENT_D PORTC = 0b11110111;
#define SEGMENT_E PORTC = 0b11101111;
#define SEGMENT_F PORTC = 0b11011111;
#define SEGMENT_G PORTC = 0b10111111;
#define SEGMENT_DP PORTC = 0b01111111;
#define SEGMENT_NONE PORTC = 0b11111111;
#define CC_DISPLAY	// Actually we use common anode display but as we mux
// segments and not digits we must use inverted patterns!
 #endif

//#opt compress
// #include <16F1788.h>
//#device ADC=12;

#include "7seg_chars.h"
#include <stdlib.h>
#include <stdio.h>
#include <avr/eeprom.h>	
#define F_CPU 16000000UL
#include <util/delay.h>
#include <string.h>
// for reading / writing int16
//#fuses NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV25,NOPROTECT,NOLVP,NODEBUG,PUT,NOMCLR  
// #use delay(int=16MHz, clock=32MHz)                               
// #use FAST_IO(a) 
// #use FAST_IO(b)
// #use FAST_IO(c)
// #use FAST_IO(e)

// Some defaut parameter values to EEPROM. Not necessary since all EEPROM values are verified to be
// in range at startup.
// #rom 0xf000 = {0x7c,0x01,0xfa,0x00,0x05,0x00,0x1e,0x00}
// #rom 0xf008 = {0x00,0x00,0x00,0x08,0x05,0x00,0x00,0x00}
// #rom 0xf010 = {0x32}

#define REED_PULLUP 0x10;          //0x04;                            // PIN_B3, jetzt PA4, wieder gndert auf PB3
#define HEATER_1    0x10;                            // PIN_B4
#define HEATER_2    0x20;                            // PIN_B5

#define VREF_ON     0x02;                            // PIN_D1
#define TRUE 1
#define FALSE 0


// Assign some 'factory default' values to use when run for the first time
// These are also used if any of the parameters stored in eeprom are outside
// of valid range
#define DEFAULT_SETPOINT 350     //380
#define DEFAULT_SETBACK 250
#define DEFAULT_SETBACK_DELAY 5
#define DEFAULT_POWEROFF_DELAY 30
#define DEFAULT_TEMPERATURE_OFFSET 0
#define DEFAULT_REFERENCE 2560                    //2048
#define DEFAULT_STEPSIZE 5
#define DEFAULT_TEMPERATURE_UNIT DISP_C
#define DEFAULT_POORMODE 0
#define DEFAULT_MAINSFREQUENCY 50
#define DEFAULT_MAXDUTY 5				// 58% duty cycle on 50 Hz, 50% on 60 Hz
#define DEFAULT_IDLEDUTY 0				// Idle detection from duty cycle off as default

// Assign sane / safe limits to the above parameters
#define MAX_SETPOINT 450				// these are also used
#define MIN_SETPOINT 100				// for setback range
#define MAX_SETBACK_DELAY 30
#define MAX_POWEROFF_DELAY 120
#define MAX_TEMPERATURE_OFFSET 40
#define MAX_REFERENCE_TOLERANCE 128		// max. tolerance is 5% of 2.048 (urspr: 103) (2,56/20)
#define MAX_MAXDUTY 9
#define MIN_MAXDUTY 3					// This means actually max setting, which is 75% for 50 Hz and 70% for 60 Hz
#define MAX_IDLEDUTY 75

// Message examples
int8_t const  cold[5] = {_C,_O,_L,_D,_SPACE}; //  CoLd
int8_t const  stby[5] = {_S,_T,_B,_Y,_SPACE}; //  Stby
int8_t const  setb[5] = {_S,_E,_T,_B,_SPACE}; //  SEtb
int8_t const  bacc[5] = {_B,_A,_C,_C,_SPACE}; //  bAcc
int8_t const  dela[5] = {_D,_E,_L,_A,_SPACE}; //  dELA
int8_t const   off[5] = {_SPACE,_O,_F,_F,_SPACE}; //  OFF
int8_t const  poff[5] = {_P,_O,_F,_F,_SPACE}; //  POFF
int8_t const  ofse[5] = {_O,_F,_S,_E,_SPACE}; //  oFSE
int8_t const  unit[5] = {_U,_N,_I,_T,_SPACE}; //  Unit
int8_t const  step[5] = {_S,_T,_E,_P,_SPACE}; //  StEP
int8_t const  diag[5] = {_D,_I,_A,_G,_SPACE}; //  diAG
int8_t const   ref[5] = {_SPACE,_R,_E,_F,_SPACE}; //  rEF
int8_t const  type[5] = {_T,_Y,_P,_E,_SPACE}; //  tyPE
int8_t const  wmrp[5] = {_W,_M,_R,_P,_SPACE}; //  WMrP
int8_t const  wmrt[5] = {_W,_M,_R,_T,_SPACE}; //  WMrt
int8_t const    nc[5] = {_SPACE,_SPACE,_N,_C,_SPACE}; //  nC
int8_t const  reed[5] = {_R,_E,_E,_D,_SPACE}; //  rEEd
int8_t const  open[5] = {_O,_P,_E,_N,_SPACE}; //  oPEn
int8_t const  clos[5] = {_C,_L,_O,_S,_SPACE}; //  CLoS
int8_t const  tc_1[5] = {_T,_C,_SPACE,_1,_SPACE}; //  tC 1
int8_t const  tc_2[5] = {_T,_C,_SPACE,_2,_SPACE}; //  tC 2
int8_t const  pwm1[5] = {_D,_C,_SPACE,_1,_SPACE}; //  dC 1
int8_t const  pwm2[5] = {_D,_C,_SPACE,_2,_SPACE}; //  dC 2
int8_t const  idle[5] = {_I,_D,_L,_E,_SPACE}; //  idLE
int8_t const  dcli[5] = {_D,_C,_L,_I,_SPACE}; //  dCLi
int8_t const  poor[5] = {_P,_O,_O,_R,_SPACE}; //  Poor
int8_t const    on[5] = {_SPACE,_SPACE,_O,_N,_SPACE}; // on
int8_t const  freq[5] = {_F,_R,_E,_Q,_SPACE}; //	FrEQ
int8_t const  vers[5] = {_V,_E,_R,_S,_SPACE}; //  vErS
int8_t const  adkt[5] = {_A,_D,_K,_T}; //ADC-Wert von KTY zum debuggen
int8_t const  adre[5] = {_A,_D,_R,_E}; //ADC-Wert von Reed auslesen

int8_t const hex_table[16]= {_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F};

typedef enum {
	ST_MAIN,
	ST_SETBACK,
	ST_STANDBY,
	ST_MENU_MAIN,
	ST_MENU_SETBACK,
	ST_ADJ_SETBACK,
	ST_MENU_DELAY,
	ST_ADJ_DELAY,
	ST_MENU_POWEROFF,
	ST_ADJ_POWEROFF,
	ST_MENU_OFFSET,
	ST_ADJ_OFFSET,
	ST_MENU_UNIT,
	ST_ADJ_UNIT,
	ST_MENU_STEP_SIZE,
	ST_ADJ_STEP_SIZE,
	ST_MENU_DIAGNOSTICS,
	ST_MENU_BACK_FROM_DIAGNOSTICS,
	ST_MENU_COLD_COMPENSATION,
	ST_SHOW_COLD_COMPENSATION,
	ST_MENU_REFERENCE,
	ST_ADJ_REFERENCE,
	ST_MENU_TIP_TYPE,
	ST_SHOW_TIP_TYPE,
	ST_MENU_REED_STATE,
	ST_SHOW_REED_STATE,
	ST_MENU_TC_1_READING,
	ST_SHOW_TC_1_READING,
	ST_MENU_TC_2_READING,
	ST_SHOW_TC_2_READING,
	ST_MENU_PWM_1_READING,
	ST_SHOW_PWM_1_READING,
	ST_MENU_PWM_2_READING,
	ST_SHOW_PWM_2_READING,
	ST_MENU_IDLE_DC,
	ST_ADJ_IDLE_DC,
	ST_MENU_DC_LIMIT,
	ST_ADJ_DC_LIMIT,
	ST_MENU_POOR,
	ST_ADJ_POOR,
	ST_MENU_FREQUENCY,
	ST_ADJ_FREQUENCY,
	ST_MENU_FW_VERSION,
	ST_SHOW_FW_VERSION,
} STATES;
STATES state = ST_MAIN;

typedef enum {
	EVT_LEFT,
	EVT_RIGHT,
	EVT_BUTTON,
	EVT_LONG_PRESS,
	EVT_NONE
} EVENTS;
EVENTS event = EVT_NONE;

typedef enum {
	TYPE_WMRP,
	TYPE_WMRT,
	TYPE_NC
} TIP_TYPES;
TIP_TYPES tiptype = TYPE_WMRP;

typedef enum {
	REED_CLOSED,
	REED_OPEN
} REED_STATES;
REED_STATES reed_status = REED_OPEN;
REED_STATES prev_reed_status = REED_OPEN;

typedef enum {
	DISP_C,
	DISP_F,
	DISP_NUM,
	DISP_REF
} DISP_MODES;


//Skalierte ADC-Werte auf 10 bit, 2,56V Referenz und OP-Verstrkung 312: Wert/4*(2,048V/2,56V)*(312/241)
const uint16_t tc_lookup[51] = {0, 260, 525, 796, 1071, 1352, 1638, 1929, 2224, 2523,
	2826, 3133, 3444, 3759,	4077, 4398,	4722, 5049,	5379, 5711,
	6046, 6382,	6721, 7061, 7403, 7746,	8091, 8436, 8782, 9129,
	9477, 9824,	10172, 10520, 10867, 11214,	11561, 11906, 12251, 12594,
	12936, 13277, 13616, 13953,	14288, 14621, 14951, 15278,	15603, 15925,
	16244
};
 
 
// Custom thermocouple 0..500 C difference, op amp gain 241, reference 2.048V
// after second calibration round
// uint16_t tc_lookup[51] = {0,1003,2028,3073,4138,5223,6327,7449,8588,9743,
//	10915,12101,13302,14517,15745,16986,18238,19501,20774,22057,
//	23349,24649,25956,27271,28591,29916,31247,32581,33918,35258,
//	36600,37943,39286,40629,41971,43311,44649,45984,47315,48642,
//	49963,51279,52588,53889,55183,56468,57743,59008,60263,61506,
//62736};

// Here is table after first calibration round
//unsigned int16 ROM tc_lookup[51] = {0,944,1912,2903,3917,4954,6011,7089,8188,9305,
//									10441,11595,12767,13954,15158,16376,17609,18855,20115,21387,
//									22670,23964,25268,26582,27904,29235,30572,31917,33267,34623,
//									35983,37347,38714,40084,41455,42827,44200,45572,46943,48313,
//									49679,51043,52403,53758,55108,56452,57789,59119,60441,61753,
//									63057};


// KTY82/110 Werte Skaliert auf 10 bit: Werte/4:
const uint16_t kty_lookup[21] = {5570, 5928, 6295, 6655, 7006, 7357, 7697, 8029, 8353, 8663,
	8967, 9258, 9535, 9804,	10061, 10307, 10543, 10764,	10964, 11140, 11282};

// KTY82/110 temperature sensor connected between GND and 1k pullup to reference. -50...+150 in 10 degree steps
// ADC REF = 2.048V
// uint16_t kty_lookup[21] = {22278,23713,25181,26619,28023,29428,30787,32116,33411,34652,
// 35868,37030,38138,39216,40242,41227,42172,43054,43857,44558,45126};

 uint8_t DISPLAY[5];			// Display memory, last byte is clock colons / degree sign
 uint8_t CURRENT_SEG = 0;

 uint8_t oldenc = 0;

 int32_t temp1=0;
 int32_t temp2=0;
 uint32_t ylempi=0;
 uint32_t alempi=0;
 uint16_t adc_sum = 0;
 int16_t right_buf=0;
 int16_t left_buf=0;
 int16_t kty_buf=25;
 int16_t new_temp=0;

// Saveable parameters. These will be downloaded from EEPROM and set to default
// if any of them are out of range, therefore no need to give initial values.
 int16_t setpoint;
 int16_t setback;
 int16_t setback_delay;
 int16_t poweroff_delay;
 int16_t temperature_offset;
 int16_t reference;
 int16_t stepsize;
DISP_MODES temperature_unit;
uint8_t poorMode;
uint8_t mainsFrequency;
uint8_t maxDuty;   
uint8_t idleDuty;   

int16_t normal_setpoint=0;
int8_t i=0;
int16_t j=0;
uint16_t buttimer=0;
uint8_t releasetimer=0;
uint8_t heaterStatus=0xff;
uint8_t heater2Status=0xff;
uint32_t milliseconds=0;
uint32_t idleminutes=0;
uint16_t setpointdelay=1000;
int16_t filteredtemp=0;
uint8_t savesettings=0;
volatile uint8_t mainsCycles=0;
uint16_t heaterCycles=0;
uint16_t heater2Cycles=0;
uint8_t heaterDuty=0;
uint8_t heater2Duty=0;
uint16_t dutyCounter=0;
uint8_t degreeDimmer=0;
uint8_t digit4Temp=_SPACE;

void show_7seg_num(DISP_MODES mode,  int16_t num);						// converts integers to 7 segment numbers
void readAdc(int16_t* adc_r, const uint8_t kanal);		//const uint8_t kanal					// reads specified ADC input 16 times to get a virtual 16 bit reading
void tc_lookup_32bit(uint32_t* adc_sum,  int16_t* temperature);	        // converts thermocouple voltage to temperature
void kty_lookup_16bit(uint16_t* adc_sum,  int16_t* temperature);	    // converts cold junction compensation sensor voltage to temperature
void menuMachine();													    // updates menu state based on encoder event
void updateDisplay();	
												                        // updates display memory based on current menu state
void recognizeTypeInStand()	;											// recognizes tip type even when in stand
void saveParms();														// saves current parameters to EEPROM
void set_adc_channel(const uint8_t channel);                            //ADC Kanal setzen
uint16_t read_adc();                                                    // ADC Wert lesen 

// #int_timer2                                                             // timer2 is set to 832 us

ISR (TIMER0_COMP_vect) {
	// output_a(_8); // for testing interrupt length, currently 20-25 us when menuMachine call moved to interrupt
	
		
	// Update mainsCycles. It is used to trigger next temperature readout, somewhat synced to AC phase
	mainsCycles++;
	if (mainsFrequency==50) {
		if (mainsCycles>=24)	// 50 Hz
		mainsCycles=0;
	}
	else {
		if (mainsCycles>=20)	// 60 Hz
		mainsCycles=0;
	}
	
	// Dim decimal point if reed closed, i.e. iron on stand
	if (degreeDimmer == 1 && reed_status == REED_CLOSED)
	digit4Temp = _SPACE;
	else
	digit4Temp = DISPLAY[4];
	
		
	// Seven segment display multiplexing
	
	
	PORTD = (_SPACE & 0xF8);                       // clear anodes (urspr. output_a(_SPACE);)  // Bitmaske & 0xf8, um die anderen Pins nicht zu manipilieren
	
	switch(CURRENT_SEG) {
		case 0:	SEGMENT_A
		PORTD = ((((digit4Temp&0x01)<<7)|((DISPLAY[3]&0x01)<<6)|((DISPLAY[2]&0x01)<<5)|((DISPLAY[1]&0x01)<<4)|((DISPLAY[0]&0x01)<<3))& 0xF8);
		break;
		case 1:	SEGMENT_B
		PORTD = ((((digit4Temp&0x02)<<6)|((DISPLAY[3]&0x02)<<5)|((DISPLAY[2]&0x02)<<4)|((DISPLAY[1]&0x02)<<3)|((DISPLAY[0]&0x02)<<2))& 0xF8);
		break;
		case 2:	SEGMENT_C
		PORTD = ((((digit4Temp&0x04)<<5)|((DISPLAY[3]&0x04)<<4)|((DISPLAY[2]&0x04)<<3)|((DISPLAY[1]&0x04)<<2)|((DISPLAY[0]&0x04)<<1))& 0xF8);
		break;
		case 3:	SEGMENT_D
		PORTD = ((((DISPLAY[3]&0x08)<<3)|((DISPLAY[2]&0x08)<<2)|((DISPLAY[1]&0x08)<<1)|(DISPLAY[0]&0x08))& 0xF8);
		break;
		case 4:	SEGMENT_E
		PORTD = ((((DISPLAY[3]&0x10)<<2)|((DISPLAY[2]&0x10)<<1)|((DISPLAY[1]&0x10))|((DISPLAY[0]&0x10)>>1))& 0xF8);
		break;
		case 5:	SEGMENT_F
		PORTD = ((((DISPLAY[3]&0x20)<<1)|((DISPLAY[2]&0x20))|((DISPLAY[1]&0x20)>>1)|((DISPLAY[0]&0x20)>>2))& 0xF8);
		break;
		case 6:	SEGMENT_G
		PORTD = ((((DISPLAY[3]&0x40))|((DISPLAY[2]&0x40)>>1)|((DISPLAY[1]&0x40)>>2)|((DISPLAY[0]&0x40)>>3))& 0xF8);
		break;
		case 7:	SEGMENT_DP
		PORTD = ((((DISPLAY[3]&0x80)>>1)|((DISPLAY[2]&0x80)>>2)|((DISPLAY[1]&0x80)>>3)|((DISPLAY[0]&0x80)>>4) _ADD_HEATER_STATUS _ADD_HEATER_STATUS)& 0xF8);
		break;
		default:SEGMENT_NONE
	}
	
	// Dim the display if on standby
	CURRENT_SEG++;
	if (CURRENT_SEG==8 && state!=ST_STANDBY) {
		CURRENT_SEG=0;
		degreeDimmer = !degreeDimmer;	// Toggle degreeDimmer bit at each multiplexing round
	}
	if (CURRENT_SEG==16)
	CURRENT_SEG=0;

   PORTD |= 0b00000001; //PD0 Pullup fr Encoder Button, //hier nochmal reingeodert, da sonsst pull up von Button (PD0 nicht aktiv)

	// ENCODER READOUT
	if (!(PINB & 0x80))                                      //!input(PIN_B7)
	{
		if (!(PINB & 0x40) && (oldenc & 0x40))               //!input(PIN_B6)
		event = EVT_RIGHT;
		if ((PINB & 0x40) && !(oldenc & 0x40))               //input(PIN_B6) 
		event = EVT_LEFT;
	}
	oldenc = PINB;                                            //input_b();

	// Encoder button readout
	if (!(PIND & 0x01)||(buttimer>0&&buttimer<72))	    // 60 ms debounce on press !input(PIN_E3) Gendert auf Pin D0
	buttimer++;
	else
	buttimer=0;
	
	if ((PIND & 0x01)&&(releasetimer>0))			// 30 ms debounce on release input(PIN_E3) Gendert auf Pin D0
	releasetimer--;
	else if (!(PIND & 0x01)&&buttimer>1)                          //!input(PIN_E3)
	releasetimer=36;
	
	if (buttimer==1 && releasetimer==0)
	event = EVT_BUTTON;
	
	else if (buttimer>1023)
	event = EVT_LONG_PRESS;
	
           
	// End reading encoder
	
	menuMachine();	// menuMachine() call moved here to prevent missing events if interrupt happens during menuMachine call
	
	// Count time when iron is on stand or heater duty cycles are below idle duty.
	if (((heaterDuty < idleDuty && heater2Duty < idleDuty) || reed_status == REED_CLOSED) && (state == ST_MAIN || state == ST_SETBACK))
	milliseconds++;
	else {
		milliseconds=0;
		idleminutes=0;
	}
	
	if (milliseconds==72191)	// 72115*832 us = 60 s. But 72191 is faster to compare
	{
		milliseconds=0;
		if (idleminutes < 255)	// don't let idleminutes overflow
		idleminutes++;
	}

	// The time counted above is needed to determine when the setback or standby modes should be activated.
	if (milliseconds == 1 && idleminutes == setback_delay && setback_delay != 31 && state == ST_MAIN)
	state = ST_SETBACK;
	if (milliseconds == 1 && idleminutes == poweroff_delay && poweroff_delay != 130 && (state == ST_MAIN || state == ST_SETBACK))
	state = ST_STANDBY;

	// Setpointdelay means time setpoint is shown after adjusting, before display reverts back to show actual temperature
	if (setpointdelay>0)
	setpointdelay--;

	// The setpoint is saved to EEPROM after the setpoint delay, to save EERPOM from wearing out
	if (setpointdelay==1)
	savesettings=1;
	
	// Calculate the average duty cycle driven to heaters
	if (heaterStatus == 0x7f)
	heaterCycles++;

	if (heater2Status == 0x7f)
	heater2Cycles++;
	
	dutyCounter++;
	if (dutyCounter == 800) {
		dutyCounter = 0;
		heaterDuty = heaterCycles>>3;
		heater2Duty = heater2Cycles>>3;
		heaterCycles = 0;
		heater2Cycles = 0;
	}

	// output_a(_SPACE); // to test interrupt length
}

int main (void)
{
	 uint16_t temp_reading=0;
	 uint32_t right_sum=0;
     uint32_t left_sum=0;
	 uint32_t right_sum_2=0;
     uint32_t left_sum_2=0;
     uint16_t kty_sum=0;
	 int16_t kty_filtering_sum=0;
     int16_t signed_temp_reading=0;
	 uint16_t reed_sum=0;
	 uint8_t kty_cycles=0;
	 uint8_t kty_filt_cycles=0;
     uint8_t calculateTemp=0;
	 uint8_t checkTip=0;
     uint8_t paramsOutOfRange=0;
	                           
	
   DDRA = 0b00010000;   //Ursprnglich: 0b11111000 (weiterer ADC-Eingang ntig, da "IN STAND" bei PIC auf 12 war und jetzt auf 3 kommt
   DDRB = 0b00110000;    //0b00110000
   DDRC = 0xFF;
   DDRD = 0b11111010;   //PD0 Eingang fr Encoder Button, PD1 Ausgang fr Mosfet, das VRef von ADC schaltet. PD 3-7 Ausgang fr Digits
	 
   PORTA = 0b00000000;
   PORTB = 0b11000000;           
   PORTC = 0b00000000;
   PORTD = 0b00000111; //PD0 Pullup fr Encoder Button, PD1 am Anfang high, damit VREF Mosfet sperrt 
	 
   PORTB &= ~HEATER_1;	// Heater 1 (right) off (urspr. output_low(HEATER_1))
   PORTB &= ~HEATER_2;	// Heater 2 (left) off  (urspr. output_low(HEATER_2)) 
	
    // set timer0 to 832 s (prescaler 256, period 12) (fr 16 MHZ)
	
	TCCR0 = (1<<WGM01) ; //CTC MODE
	TCCR0 |= (1<<CS02) ; //Prescaler 256
	OCR0 = 52;  //Compare Register auf 52 dez
	TIMSK = (1<<OCIE0);   //Compare Match Interrupt freigeben
	
	sei(); //globale Interruptfreigabe
	 
	DISPLAY[4]=_SPACE;
	
    /* setup_adc_ports( sAN0|sAN1|sAN2|sAN9|sAN12,VSS_FVR);	// AN9 = reed pullup
	   setup_vref(VREF_ON | VREF_ADC_2v048 | VREF_COMP_DAC_2v048);
	   setup_opamp2(OPAMP_ENABLED | OPAMP_HIGH_GBWP_MODE | OPAMP_NI_TO_FVR);		// To enable KTY pullup
	   setup_opamp2(OPAMP_DISABLED);		// To disable KTY pullup
	   setup_dac(DAC_OFF); 

	   setup_adc(ADC_CLOCK_DIV_32); */         // Gives 1s conversion clock cycle time (fastest recommended)
	
	
	ADMUX |= (1<<REFS1) | (1<<REFS0);               //interne Spannungsreferenz 2,56V (AREF PIN 32) nehmen, externen Kondensator 
	ADCSRA |= (1<<ADEN) | (1<<ADSC) | (1<<ADATE);   //ADC enable, ADC start conversion, Auto Trigger Mode
	PORTD &= ~ VREF_ON;                              //VREF ber Mosfet nach auen schalten  (To enable KTY pullup)  
	PORTD |= VREF_ON;                                //VREF ber Mosfet wieder abschalten
	ADCSRA |= (1<<ADPS2) | (0<<ADPS1) | (0<<ADPS0); //Teilerfaktor 16 zwischen XTAL Takt und ADC Takt => 1S conversion clock


		
	setpoint = eeprom_read_word(0);
	setback = eeprom_read_word(2);			// must use even addresses because one int16 takes two bytes
	setback_delay = eeprom_read_word(4);
	poweroff_delay = eeprom_read_word(6);
	temperature_offset = eeprom_read_word(8);
	reference = eeprom_read_word(10);
	stepsize = eeprom_read_word(12);
	temperature_unit = eeprom_read_byte(14);
	poorMode = eeprom_read_byte(15);
	mainsFrequency = eeprom_read_byte(16);
	maxDuty = eeprom_read_byte(17);
	idleDuty = eeprom_read_byte(18);

	if (setpoint < MIN_SETPOINT || setpoint > MAX_SETPOINT)		// Ensure safe temperature setpoints in case of corrupt EEPROM data
	paramsOutOfRange=TRUE;
	if (setback < MIN_SETPOINT || setback > MAX_SETPOINT)
	paramsOutOfRange=TRUE;
	if (setback_delay < 0 || setback_delay > MAX_SETBACK_DELAY+1)
	paramsOutOfRange=TRUE;
	if (poweroff_delay < 0 || poweroff_delay > MAX_POWEROFF_DELAY+10)
	paramsOutOfRange=TRUE;
	if (temperature_offset < -(MAX_TEMPERATURE_OFFSET) || temperature_offset > MAX_TEMPERATURE_OFFSET)
	paramsOutOfRange=TRUE;
	if (reference < 2560 - MAX_REFERENCE_TOLERANCE || reference > 2560 + MAX_REFERENCE_TOLERANCE)    //2048
	paramsOutOfRange=TRUE;
	if (stepsize != 1 && stepsize != 5)
	paramsOutOfRange=TRUE;
	if (temperature_unit != DISP_C && temperature_unit != DISP_F)
	paramsOutOfRange=TRUE;
	if (mainsFrequency != 50 && mainsFrequency != 60)
	paramsOutOfRange=TRUE;
	if (maxDuty < MIN_MAXDUTY || maxDuty > MAX_MAXDUTY)
	paramsOutOfRange=TRUE;
	if (idleDuty > MAX_IDLEDUTY)
	paramsOutOfRange=TRUE;
	
	if (paramsOutOfRange) {
		setpoint = DEFAULT_SETPOINT;
		setback = DEFAULT_SETBACK;
		setback_delay = DEFAULT_SETBACK_DELAY;
		poweroff_delay = DEFAULT_POWEROFF_DELAY;
		temperature_offset = DEFAULT_TEMPERATURE_OFFSET;
		reference = DEFAULT_REFERENCE;
		stepsize = DEFAULT_STEPSIZE;
		temperature_unit = DEFAULT_TEMPERATURE_UNIT;
		poorMode = DEFAULT_POORMODE;
		mainsFrequency = DEFAULT_MAINSFREQUENCY;
		maxDuty = DEFAULT_MAXDUTY;
		idleDuty = DEFAULT_IDLEDUTY;
	}

	normal_setpoint = setpoint;

	recognizeTypeInStand();
	recognizeTypeInStand();
	_delay_ms(10);	// let auto-zero op-amps stabilize before tip type recognition
	tiptype = TYPE_WMRP;
	recognizeTypeInStand();

   

	/* Infinite loop */
	while(1)
	
	{   
				
		PORTB &= ~ HEATER_1;	// Heater 1 (right) off (urspr.: output_low(HEATER_1))
		PORTB &= ~ HEATER_2;	// Heater 2 (left) off (urspr.: output_low(HEATER_2))
		heaterStatus = 0xff;
		heater2Status = 0xff;
				
		_delay_us(10);

		// READ REED STATUS - Reed pullup is on for 35 us (measured 21.1.2017 / FW 0.6)
		ADMUX = (0<<REFS1) | (1<<REFS0);               //fr externe Spannungsreferenz an AVCC (gebrckt auf 5V)
		//ADCSRA |= (1<<ADEN) | (1<<ADSC) | (1<<ADATE);   
		DDRA |= REED_PULLUP;                         
		PORTA |= REED_PULLUP;
		set_adc_channel(3);                              //Auf 3 gendert (urspr. 12)
	    _delay_us(60); //_   15   bei 50 flackert der typ noch         
		reed_sum = read_adc();
		
		
		//_delay_ms(100);
		  
		PORTA &= ~REED_PULLUP;
		DDRA &= ~REED_PULLUP;              
		                                       
		                                                
		ADMUX = (1<<REFS1) | (1<<REFS0);                //fr interne Spannungsreferenz 2,56V
		
		//_delay_us(5);
														  
		// KTY READOUT - KTY pullup is on for 34 us (measured 21.1.2017 / FW 0.6)
		PORTD &= ~VREF_ON;                                                               //VREF ber Mosfet nach auen schalten   ursprnglich:  //setup_opamp2(OPAMP_ENABLED | OPAMP_HIGH_GBWP_MODE | OPAMP_NI_TO_FVR);		// To enable KTY pullup
		set_adc_channel(2);
		_delay_us(30);//_delay_ms(500);  //5
		kty_sum += read_adc();
		
		//show_7seg_num(DISP_NUM, kty_sum);       //Inhalt ADC-Register berprfen
		if (state != ST_ADJ_REFERENCE)			// Keep KTY pullup on (reference on) when adjusting reference compensation
		  PORTD |= VREF_ON;                      //VREF ber Mosfet wieder abschalten   ursprnglich:   //setup_opamp2(OPAMP_DISABLED);		// To disable KTY pullup
        
		
       
		//_delay_us(300);		// takes about 200-300 s for opamp output to stabilize after pullups turned off - delay not
		// needed anymore because wait for mainsCycles below gives more than enough time.
		
		if (mainsFrequency==50)										// added 2*832us waiting to reduce heating duty cycle in FW 0.7
		while (mainsCycles != maxDuty && mainsCycles != 12+maxDuty) {	}		// wait here for measurement cycle start (50 Hz)
		else
		while (mainsCycles != maxDuty && mainsCycles != 10+maxDuty) {	}		// wait here for measurement cycle start (60 Hz)
      		
		 		
	   

		if (mainsCycles == maxDuty) {	
			readAdc(&left_sum, 1);    // this function should take about 0.3 ms (ADC_CLOCK_DIV32 @ 32 MHz)
			readAdc(&right_sum, 0);
		}
		
		
		else {
			readAdc(&left_sum_2, 1);
			readAdc(&right_sum_2, 0);
			calculateTemp = 1;
			
		}
		                
		if (checkTip) {				// seems that new iron is connected with reed closed - lets check the type!
			tiptype = TYPE_WMRP;	// defalut to WMRP
			recognizeTypeInStand();	// check if it is WMRT
			checkTip=0;
		}
		
		if (tiptype == TYPE_NC) {
			right_buf=555;
		}

		if (right_buf<setpoint) {
			heaterStatus = 0x7f;  //
			PORTB|= HEATER_1;                          // output_high(HEATER_1);
		}
		else {
			heaterStatus = 0xff;
		}

		if (left_buf<setpoint && tiptype == TYPE_WMRT) {
			heater2Status = 0x7f;   //
			PORTB|= HEATER_2;                           // output_high(HEATER_2);
		}
		else {
			heater2Status = 0xff;
		}

		
		kty_cycles++;
		if (kty_cycles==16) {
			kty_lookup_16bit(&kty_sum, &signed_temp_reading);
			kty_sum=0;
			if (signed_temp_reading > 130)	// No KTY sensor or bad contact
			signed_temp_reading=30;		// use some default value
			kty_filtering_sum += signed_temp_reading;
			kty_filt_cycles++;
			if (kty_filt_cycles==16) {
				kty_filt_cycles=0;
				kty_buf = kty_filtering_sum / 16;
				kty_filtering_sum=0;
			}
			kty_cycles=0;
		}

		if (calculateTemp==1) {				// takes about 1.5ms
			left_sum = (left_sum + left_sum_2)/2;
			right_sum = (right_sum + right_sum_2)/2;
			tc_lookup_32bit(&left_sum, &left_buf);
			tc_lookup_32bit(&right_sum, &right_buf);
			calculateTemp=0;
		}

		if (savesettings && state == ST_MAIN)	// do not save setpoint if already gone to stby
		{
			eeprom_write_word(0, setpoint);
			savesettings=0;
		}
		
		
		
		// some filtering to displayed temperature to reduce flickering
		temp_reading = filteredtemp*30;
		temp_reading = temp_reading + right_buf*10;
		filteredtemp = temp_reading / 40;

		if ((filteredtemp-setpoint < 3) && (setpoint-filteredtemp < 3))
		filteredtemp = setpoint;

		if (reed_sum<78) {                                    //312
			if (poorMode) {
				tiptype = TYPE_WMRP;
				reed_status = REED_OPEN;
			}
			else if (tiptype == TYPE_NC) {
				checkTip=1;
				reed_status = REED_CLOSED;
			}
			else
			reed_status = REED_CLOSED;
		}
		else if (reed_sum>450 && reed_sum<575) {           //1798      2298
			tiptype = TYPE_WMRP;
			reed_status = REED_OPEN;
		}
		else if (reed_sum>620 && reed_sum<745) {           //2481       2981
			tiptype = TYPE_WMRT;
			reed_status = REED_OPEN;
		}
		else if (reed_sum>938) {                           //3750
			tiptype = TYPE_NC;
			reed_status = REED_OPEN;
		}
      
		updateDisplay();		// updateDisplay call takes about 520 s when in 'normal' mode (displaying temp)
		                         
				  
		if (mainsFrequency==50)
		while (mainsCycles != 0 && mainsCycles != 12) {	}		// wait here for heating cycle to stop (50 Hz)
		else
		while (mainsCycles != 0 && mainsCycles != 10) {	}		// wait here for heating cycle to stop (60 Hz)
	
	}	

}

void show_7seg_num(DISP_MODES mode,  int16_t num)
{
	 int8_t j;                       // Digit index
	 int16_t temp16;

	switch(mode) {
		case DISP_C:
		DISPLAY[3] = _CELSIUS;
		DISPLAY[4] = _DEGREE;
		j = 2;
		break;
		case DISP_F:
		DISPLAY[3] = _F;
		DISPLAY[4] = _DEGREE;
		num = num*9/5;
		if (state != ST_ADJ_OFFSET && state != ST_ADJ_STEP_SIZE)
		num = num+32;
		if (num > 999)
		num = 999;
		j = 2;
		break;
		case DISP_NUM:
		case DISP_REF:
		DISPLAY[4] = _SPACE;
		j = 3;
	}

	temp16 = labs(num);

	DISPLAY[j] = hex_table[temp16 %10];
	while (j > 0)
	{
		j--;
		if (j==0 && mode == DISP_REF)
		DISPLAY[j] = hex_table[(temp16/10) %10];// _ADDPOINT;
		else if ((temp16/10) == 0 && num>=0)
		DISPLAY[j] = _SPACE;
		else if ((temp16/10) == 0 && temp16 != 0 && num < 0)
		DISPLAY[j] = _MINUS;
		else if (temp16 == 0 && num < 0)
		DISPLAY[j] = _SPACE;
		else
		DISPLAY[j] = hex_table[(temp16/10) %10];
		temp16 /= 10;
	}
}

void readAdc(int16_t* adc_r, const uint8_t kanal) {                                      //const uint8_t kanal
	set_adc_channel(kanal);                                       //testweise auskommentiert, da Kanalwahl nicht funktioniert
	_delay_us(30);    //5   25 mindestens ntig, sonst flackert Temperaturanzeige
	i=0;
	adc_sum = 0;
	while (i<16) { // sample 16 times to get a virtual 16 bit reading..
		adc_sum += read_adc();
		i++;
	}
	*adc_r = adc_sum;
}

void tc_lookup_32bit(uint32_t* adc_sum, int16_t* temperature)
{
	uint32_t sum32;
	// sum32 = (unsigned int32)*adc_sum * (unsigned int32)reference / 2048;	// compensate for reference inaccuracy
	sum32 = *adc_sum * (uint32_t)reference /2560 ;	// compensate for reference inaccuracy (urspr.: / 2048)
	for (j=0;j<50;j++)
	{
		if (sum32<tc_lookup[j])
		break;
	}
	
	ylempi = tc_lookup[j];
	alempi = tc_lookup[j-1];
	temp1=10*(alempi-sum32);
	temp2=(alempi-ylempi);
	new_temp = temp1/temp2+ 10*(j-1)+kty_buf;		// -1 because tc starts from zero.
	new_temp = new_temp - temperature_offset;		// subtract offset setting
	
	*temperature = new_temp;
}

void kty_lookup_16bit(uint16_t* adc_sum, int16_t* temperature)
{
	for (j=0;j<21;j++)
	{
		if (*adc_sum<kty_lookup[j])
		break;
	}
	
	ylempi = kty_lookup[j];
	alempi = kty_lookup[j-1];
	temp1=10*(alempi-*adc_sum);
	temp2=(alempi-ylempi);
	new_temp = temp1/temp2+ 10*(j-6); // -6 because kty_lookup begins from -50
	*temperature = new_temp;
}

void menuMachine()		// Takes now 3..4us when display updates moved to separate function
{
	switch(state)
	{
		case ST_MAIN:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_STANDBY;
			break;
			case EVT_LONG_PRESS:
			state = ST_MENU_MAIN;
			break;
			case EVT_LEFT:
			setpoint-=stepsize;
			if (setpoint < MIN_SETPOINT)
			setpoint = MIN_SETPOINT;
			normal_setpoint = setpoint;
			setpointdelay=1202;
			break;
			case EVT_RIGHT:
			setpoint+=stepsize;
			if (setpoint > MAX_SETPOINT)
			setpoint = MAX_SETPOINT;
			normal_setpoint = setpoint;
			setpointdelay=1202;
			break;
		}
		break;
		case ST_STANDBY:
		setpoint=0;
		if (reed_status == REED_OPEN && prev_reed_status == REED_CLOSED)
		//			if (heaterDuty > idleDuty || heater2Duty > idleDuty || (reed_status == REED_OPEN && prev_reed_status == REED_CLOSED))
		event = EVT_BUTTON;
		prev_reed_status = reed_status;
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MAIN;
			setpoint = normal_setpoint;
			milliseconds = 0;
			idleminutes = 0;
			break;
			case EVT_LONG_PRESS:
			state = ST_MENU_MAIN;
			setpoint = normal_setpoint;
			break;
		}
		break;
		case ST_SETBACK:
		if (setback < setpoint)
		setpoint = setback;
		//			if (reed_status == REED_OPEN && prev_reed_status == REED_CLOSED)
		if ((idleDuty != 0 && (heaterDuty > idleDuty || heater2Duty > idleDuty)) || (reed_status == REED_OPEN && prev_reed_status == REED_CLOSED))
		event = EVT_BUTTON;
		prev_reed_status = reed_status;
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MAIN;
			setpoint = normal_setpoint;
			milliseconds = 0;
			idleminutes = 0;
			if (setback_delay == 0)			// workaround to briefly activate ST_MAIN if setback is zero
			milliseconds = 4294961286;	// this number will overflow in 5 seconds
			break;
			case EVT_LONG_PRESS:
			state = ST_MENU_MAIN;
			setpoint = normal_setpoint;
			break;
		}
		break;
		case ST_MENU_MAIN:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MAIN;
			break;
			case EVT_LEFT:
			state = ST_MENU_DIAGNOSTICS;
			break;
			case EVT_RIGHT:
			state = ST_MENU_SETBACK;
			break;
		}
		break;
		case ST_MENU_SETBACK:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_SETBACK;
			break;
			case EVT_LEFT:
			state = ST_MENU_MAIN;
			break;
			case EVT_RIGHT:
			state = ST_MENU_DELAY;
			break;
		}
		break;
		case ST_ADJ_SETBACK:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_SETBACK;
			break;
			case EVT_LEFT:
			if(setback > MIN_SETPOINT)
			setback-=10;
			break;
			case EVT_RIGHT:
			if(setback < MAX_SETPOINT)
			setback+=10;
			break;
		}
		break;
		case ST_MENU_DELAY:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_DELAY;
			break;
			case EVT_LEFT:
			state = ST_MENU_SETBACK;
			break;
			case EVT_RIGHT:
			state = ST_MENU_POWEROFF;
			break;
		}
		break;
		case ST_ADJ_DELAY:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_DELAY;
			break;
			case EVT_LEFT:
			if(setback_delay>0)
			setback_delay--;
			break;
			case EVT_RIGHT:
			if(setback_delay < MAX_SETBACK_DELAY+1)
			setback_delay++;
			break;
		}
		break;
		case ST_MENU_POWEROFF:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_POWEROFF;
			break;
			case EVT_LEFT:
			state = ST_MENU_DELAY;
			break;
			case EVT_RIGHT:
			state = ST_MENU_OFFSET;
			break;
		}
		break;
		case ST_ADJ_POWEROFF:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_POWEROFF;
			break;
			case EVT_LEFT:
			poweroff_delay-=10;
			if (poweroff_delay<0)
			poweroff_delay=0;
			break;
			case EVT_RIGHT:
			poweroff_delay+=10;
			if (poweroff_delay > MAX_POWEROFF_DELAY)
			poweroff_delay = MAX_POWEROFF_DELAY+10;
			break;
		}
		break;
		case ST_MENU_OFFSET:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_OFFSET;
			break;
			case EVT_LEFT:
			state = ST_MENU_POWEROFF;
			break;
			case EVT_RIGHT:
			state = ST_MENU_UNIT;
			break;
		}
		break;
		case ST_ADJ_OFFSET:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_OFFSET;
			break;
			case EVT_LEFT:
			if(temperature_offset > -(MAX_TEMPERATURE_OFFSET))
			temperature_offset--;
			break;
			case EVT_RIGHT:
			if(temperature_offset < MAX_TEMPERATURE_OFFSET)
			temperature_offset++;
			break;
		}
		break;
		case ST_MENU_UNIT:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_UNIT;
			break;
			case EVT_LEFT:
			state = ST_MENU_OFFSET;
			break;
			case EVT_RIGHT:
			state = ST_MENU_STEP_SIZE;
			break;
		}
		break;
		case ST_ADJ_UNIT:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_UNIT;
			break;
			case EVT_LEFT:
			case EVT_RIGHT:
			if(temperature_unit == DISP_C)
			temperature_unit = DISP_F;
			else
			temperature_unit = DISP_C;
			break;
		}
		break;
		case ST_MENU_STEP_SIZE:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_STEP_SIZE;
			break;
			case EVT_LEFT:
			state = ST_MENU_UNIT;
			break;
			case EVT_RIGHT:
			state = ST_MENU_DIAGNOSTICS;
			break;
		}
		break;
		case ST_ADJ_STEP_SIZE:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_STEP_SIZE;
			break;
			case EVT_LEFT:
			case EVT_RIGHT:
			if(stepsize == DEFAULT_STEPSIZE)
			stepsize = 1;
			else
			stepsize = DEFAULT_STEPSIZE;
			break;
		}
		break;
		case ST_MENU_DIAGNOSTICS:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_BACK_FROM_DIAGNOSTICS;
			break;
			case EVT_LEFT:
			state = ST_MENU_STEP_SIZE;
			break;
			case EVT_RIGHT:
			state = ST_MENU_MAIN;
			break;
		}
		break;
		case ST_MENU_BACK_FROM_DIAGNOSTICS:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_DIAGNOSTICS;
			break;
			case EVT_LEFT:
			state = ST_MENU_FW_VERSION;
			break;
			case EVT_RIGHT:
			state = ST_MENU_COLD_COMPENSATION;
			break;
		}
		break;
		case ST_MENU_COLD_COMPENSATION:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_COLD_COMPENSATION;
			break;
			case EVT_LEFT:
			state = ST_MENU_BACK_FROM_DIAGNOSTICS;
			break;
			case EVT_RIGHT:
			state = ST_MENU_REFERENCE;
			break;
		}
		break;
		case ST_SHOW_COLD_COMPENSATION:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_COLD_COMPENSATION;
			break;
		}
		break;
		case ST_MENU_REFERENCE:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_REFERENCE;
			break;
			case EVT_LEFT:
			state = ST_MENU_COLD_COMPENSATION;
			break;
			case EVT_RIGHT:
			state = ST_MENU_TIP_TYPE;
			break;
		}
		break;
		case ST_ADJ_REFERENCE:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_REFERENCE;
			break;
			case EVT_LEFT:
			if(reference > 2560 - MAX_REFERENCE_TOLERANCE)     //2048
			reference--;
			break;
			case EVT_RIGHT:
			if(reference < 2560 + MAX_REFERENCE_TOLERANCE)     //2048
			reference++;
			break;
		}
		break;
		case ST_MENU_TIP_TYPE:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_TIP_TYPE;
			break;
			case EVT_LEFT:
			state = ST_MENU_REFERENCE;
			break;
			case EVT_RIGHT:
			state = ST_MENU_REED_STATE;
			break;
		}
		break;
		case ST_SHOW_TIP_TYPE:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_TIP_TYPE;
			break;
		}
		break;
		case ST_MENU_REED_STATE:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_REED_STATE;
			break;
			case EVT_LEFT:
			state = ST_MENU_TIP_TYPE;
			break;
			case EVT_RIGHT:
			state = ST_MENU_TC_1_READING;
			break;
		}
		break;
		case ST_SHOW_REED_STATE:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_REED_STATE;
			break;
		}
		break;
		case ST_MENU_TC_1_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_TC_1_READING;
			break;
			case EVT_LEFT:
			state = ST_MENU_REED_STATE;
			break;
			case EVT_RIGHT:
			state = ST_MENU_TC_2_READING;
			break;
		}
		break;
		case ST_SHOW_TC_1_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_TC_1_READING;
			break;
		}
		break;
		case ST_MENU_TC_2_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_TC_2_READING;
			break;
			case EVT_LEFT:
			state = ST_MENU_TC_1_READING;
			break;
			case EVT_RIGHT:
			state = ST_MENU_PWM_1_READING;
			break;
		}
		break;
		case ST_SHOW_TC_2_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_TC_2_READING;
			break;
		}
		break;
		case ST_MENU_PWM_1_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_PWM_1_READING;
			break;
			case EVT_LEFT:
			state = ST_MENU_TC_2_READING;
			break;
			case EVT_RIGHT:
			state = ST_MENU_PWM_2_READING;
			break;
		}
		break;
		case ST_SHOW_PWM_1_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_PWM_1_READING;
			break;
		}
		break;
		case ST_MENU_PWM_2_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_PWM_2_READING;
			break;
			case EVT_LEFT:
			state = ST_MENU_PWM_1_READING;
			break;
			case EVT_RIGHT:
			state = ST_MENU_IDLE_DC;
			break;
		}
		break;
		case ST_SHOW_PWM_2_READING:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_PWM_2_READING;
			break;
		}
		break;
		case ST_MENU_IDLE_DC:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_IDLE_DC;
			break;
			case EVT_LEFT:
			state = ST_MENU_PWM_2_READING;
			break;
			case EVT_RIGHT:
			state = ST_MENU_DC_LIMIT;
			break;
		}
		break;
		case ST_ADJ_IDLE_DC:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_IDLE_DC;
			break;
			case EVT_LEFT:
			if(idleDuty>0)
			idleDuty--;
			break;
			case EVT_RIGHT:
			if(idleDuty < MAX_IDLEDUTY)
			idleDuty++;
			break;
		}
		break;
		case ST_MENU_DC_LIMIT:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_DC_LIMIT;
			break;
			case EVT_LEFT:
			state = ST_MENU_IDLE_DC;
			break;
			case EVT_RIGHT:
			state = ST_MENU_POOR;
			break;
		}
		break;
		case ST_ADJ_DC_LIMIT:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_DC_LIMIT;
			break;
			case EVT_LEFT:
			if(maxDuty < MAX_MAXDUTY)
			maxDuty++;
			break;
			case EVT_RIGHT:
			if(maxDuty > MIN_MAXDUTY)
			maxDuty--;
			break;
		}
		break;
		case ST_MENU_POOR:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_POOR;
			break;
			case EVT_LEFT:
			state = ST_MENU_DC_LIMIT;
			break;
			case EVT_RIGHT:
			state = ST_MENU_FREQUENCY;
			break;
		}
		break;
		case ST_ADJ_POOR:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_POOR;
			break;
			case EVT_LEFT:
			case EVT_RIGHT:
			poorMode = !poorMode;
			break;
		}
		break;
		case ST_MENU_FREQUENCY:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_ADJ_FREQUENCY;
			break;
			case EVT_LEFT:
			state = ST_MENU_POOR;
			break;
			case EVT_RIGHT:
			state = ST_MENU_FW_VERSION;   
			break;
		}
		break;
		case ST_ADJ_FREQUENCY:
		switch(event)
		{
			case EVT_BUTTON:
			saveParms();
			state = ST_MENU_FREQUENCY;
			break;
			case EVT_LEFT:
			case EVT_RIGHT:
			if (mainsFrequency==50)
			mainsFrequency=60;
			else
			mainsFrequency=50;
			break;
		}
		break;
		case ST_MENU_FW_VERSION:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_SHOW_FW_VERSION;
			break;
			case EVT_LEFT:
			state = ST_MENU_FREQUENCY;
			break;
			case EVT_RIGHT:
			state = ST_MENU_BACK_FROM_DIAGNOSTICS;
			break;
		}
		break;
		case ST_SHOW_FW_VERSION:
		switch(event)
		{
			case EVT_BUTTON:
			state = ST_MENU_FW_VERSION;
			break;
		}
		break;                                              
		} // end menu case
	event = EVT_NONE;
}

void updateDisplay()	// takes 520 us when showing iron temp, 400us when KTY temp, 680us when reference, 500us when number
{
		
	switch(state)
	{
		case ST_MAIN:
		if (tiptype == TYPE_NC)                        
		memcpy (DISPLAY, nc, sizeof (DISPLAY));
		else if (setpointdelay)
		show_7seg_num(temperature_unit, setpoint);
		else
		show_7seg_num(temperature_unit,filteredtemp);
		break;
		case ST_STANDBY:
		memcpy (DISPLAY, stby, sizeof (DISPLAY));
		break;
		case ST_SETBACK:
		if (tiptype == TYPE_NC)
		memcpy (DISPLAY, nc, sizeof (DISPLAY));
		else
		show_7seg_num(temperature_unit, filteredtemp);
		break;
		case ST_MENU_MAIN:
		memcpy (DISPLAY, bacc, sizeof (DISPLAY));
		break;
		case ST_MENU_SETBACK:
		memcpy (DISPLAY, setb, sizeof (DISPLAY));
		break;
		case ST_ADJ_SETBACK:
		show_7seg_num(temperature_unit, setback);
		break;
		case ST_MENU_DELAY:
		memcpy (DISPLAY, dela, sizeof (DISPLAY));
		break;
		case ST_ADJ_DELAY:
		if (setback_delay==31)
		memcpy (DISPLAY, off, sizeof (DISPLAY));
		else
		show_7seg_num(DISP_NUM, setback_delay);
		break;
		case ST_MENU_POWEROFF:
		memcpy (DISPLAY, poff, sizeof (DISPLAY));
		break;
		case ST_ADJ_POWEROFF:
		if (poweroff_delay > 120)
		memcpy (DISPLAY, off, sizeof (DISPLAY));
		else
		show_7seg_num(DISP_NUM, poweroff_delay);
		break;
		case ST_MENU_OFFSET:
		memcpy (DISPLAY, ofse, sizeof (DISPLAY));
		break;
		case ST_ADJ_OFFSET:
		show_7seg_num(temperature_unit, temperature_offset);
		break;
		case ST_MENU_UNIT:
		memcpy (DISPLAY, unit, sizeof (DISPLAY));
		break;
		case ST_ADJ_UNIT:
		show_7seg_num(temperature_unit, 0);
		break;
		case ST_MENU_STEP_SIZE:
		memcpy (DISPLAY, step, sizeof (DISPLAY));
		break;
		case ST_ADJ_STEP_SIZE:
		show_7seg_num(temperature_unit, stepsize);
		break;
		case ST_MENU_DIAGNOSTICS:
		memcpy (DISPLAY, diag, sizeof (DISPLAY));
		break;
		case ST_MENU_BACK_FROM_DIAGNOSTICS:
		memcpy (DISPLAY, bacc, sizeof (DISPLAY));
		break;
		case ST_MENU_COLD_COMPENSATION:
		memcpy (DISPLAY, cold, sizeof (DISPLAY));
		break;
		case ST_SHOW_COLD_COMPENSATION:
		show_7seg_num(temperature_unit, kty_buf);
		break;
		case ST_MENU_REFERENCE:
		memcpy (DISPLAY, ref, sizeof (DISPLAY));
		break;
		case ST_ADJ_REFERENCE:
		show_7seg_num(DISP_REF, reference);
		break;
		case ST_MENU_TIP_TYPE:
		memcpy (DISPLAY, type, sizeof (DISPLAY));
		break;
		case ST_SHOW_TIP_TYPE:
		switch(tiptype)
		{
			case TYPE_WMRP:
			memcpy (DISPLAY, wmrp, sizeof(DISPLAY));
			break;
			case TYPE_WMRT:
			memcpy (DISPLAY, wmrt, sizeof(DISPLAY));
			break;
			case TYPE_NC:
			memcpy (DISPLAY, nc, sizeof(DISPLAY));
			break;
		}
		break;
		case ST_MENU_REED_STATE:
		memcpy (DISPLAY, reed, sizeof (DISPLAY));
		break;
		case ST_SHOW_REED_STATE:
		switch(reed_status)
		{
			case REED_OPEN:
			memcpy (DISPLAY, open, sizeof(DISPLAY));
			break;
			case REED_CLOSED:
			memcpy (DISPLAY, clos, sizeof(DISPLAY));
			break;
		}
		break;
		case ST_MENU_TC_1_READING:
		memcpy (DISPLAY, tc_1, sizeof (DISPLAY));
		break;
		case ST_SHOW_TC_1_READING:
		show_7seg_num(temperature_unit, right_buf);
		break;
		case ST_MENU_TC_2_READING:
		memcpy (DISPLAY, tc_2, sizeof (DISPLAY));
		break;
		case ST_SHOW_TC_2_READING:
		show_7seg_num(temperature_unit, left_buf);
		break;
		case ST_MENU_PWM_1_READING:
		memcpy (DISPLAY, pwm1, sizeof (DISPLAY));
		break;
		case ST_SHOW_PWM_1_READING:
		show_7seg_num(DISP_NUM, heaterDuty);
		break;
		case ST_MENU_PWM_2_READING:
		memcpy (DISPLAY, pwm2, sizeof (DISPLAY));
		break;
		case ST_SHOW_PWM_2_READING:
		show_7seg_num(DISP_NUM, heater2Duty);
		break;
		case ST_MENU_IDLE_DC:
		memcpy (DISPLAY, idle, sizeof (DISPLAY));
		break;
		case ST_ADJ_IDLE_DC:
		if (idleDuty == 0)
		memcpy (DISPLAY, off, sizeof (DISPLAY));
		else
		show_7seg_num(DISP_NUM, idleDuty);
		break;
		case ST_MENU_DC_LIMIT:
		memcpy (DISPLAY, dcli, sizeof (DISPLAY));
		break;
		case ST_ADJ_DC_LIMIT:
		if (mainsFrequency == 50)
		//				show_7seg_num(DISP_NUM, 100-maxDuty*8);		// approximation with int8 arithmetics
		show_7seg_num(DISP_NUM, (1200-(uint16_t)(maxDuty)*100)/12);
		else
		show_7seg_num(DISP_NUM, 100-maxDuty*10);
		break;
		case ST_MENU_POOR:
		memcpy (DISPLAY, poor, sizeof (DISPLAY));
		break;
		case ST_ADJ_POOR:
		if (poorMode)
		memcpy (DISPLAY, on, sizeof (DISPLAY));
		else
		memcpy (DISPLAY, off, sizeof (DISPLAY));
		break;
		case ST_MENU_FREQUENCY:
		memcpy (DISPLAY, freq, sizeof (DISPLAY));
		break;
		case ST_ADJ_FREQUENCY:
		show_7seg_num(DISP_NUM, mainsFrequency);
		break;
		case ST_MENU_FW_VERSION:
		memcpy (DISPLAY, vers, sizeof (DISPLAY));
		break;
		case ST_SHOW_FW_VERSION:
		show_7seg_num(DISP_REF, (int16_t)FW_VERSION);
		break;
	} // end display case
}

void recognizeTypeInStand()
{
	// Tip type recognision by pulsing current to heater2 = wmrt left heater. If drives TC2 to rail, must be WMRT
	// If the iron is on stand, cannot recognize based on resistor in parallel with reed because reed is closed.
	// Since FW 0.8 this is only used when necessary to recognize if WMRT is connected. Older FW used this method
	// continuously which caused continuous buzz noise from WMRT tip. The buzz comes from the heating elements of
	// WMRT when power is switched on and off rapidly. Original Weller stations don't make the noise because they
	// drive the elements with AC

	set_adc_channel(1);
	PORTB |= HEATER_2;                                                                         // output_high(HEATER_2)
	//_delay_us(20);
	uint16_t temp_reading = read_adc();
	PORTB &= ~HEATER_2;                                                                            // output_low(HEATER_2);
	if (temp_reading > 1000)                                                                                            //(temp_reading > 4000)
	tiptype = TYPE_WMRT;
}

void saveParms()
{
	PORTB &= ~HEATER_1;                             // output_low(HEATER_1);	// Turn heaters off before saving parameters - interrupts are disabled
	PORTB &= ~HEATER_2;                             //  output_low(HEATER_2);	// during EEPROM write and we don't want something bad to happen if write jams
	PORTA &= (_SPACE);                              //output_a(_SPACE); 		// clear anodes to eliminate bright flash of currently active segments
	eeprom_write_word(0, setpoint);                 //write_int16_eeprom(0, setpoint);
	eeprom_write_word(2, setback);
	eeprom_write_word(4, setback_delay);
	eeprom_write_word(6, poweroff_delay);
	eeprom_write_word(8, temperature_offset);
	eeprom_write_word(10, reference);
	eeprom_write_word(12, stepsize);
	eeprom_write_byte(14, temperature_unit);
	eeprom_write_byte(15, poorMode);
	eeprom_write_byte(16, mainsFrequency);
	eeprom_write_byte(17, maxDuty);
	eeprom_write_byte(18, idleDuty);
}

void set_adc_channel(const uint8_t channel)
{
	ADMUX = (ADMUX & 0xF8)|(channel & 0x07);  	
}

uint16_t read_adc()
{
	return ADC;
}